// Gradients shader
// Ernst Hot, July 11. 2006
//
// thorsten@oerts.dk

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = clamp;
    AddressV = clamp;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4 c1  : COLOR <String uiname="Color 1";>  = {1, 0, 0, 1};
float4 c2  : COLOR <String uiname="Color 2";>  = {1, 0, 0, 1};

 // Linear amount
float Lamn
<
    string uiname="Linear";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

 // Linear amount
float Lpow
<
    string uiname="Linear Pow";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

// Radial amount
float Ramn
<
    string uiname="Radial";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

 // Radial Pow
float Rpow
<
    string uiname="Radial Pow";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

// Radial2 amount
float R2amn
<
    string uiname="Radial2";
    float uimin=0.0;
    float uimax=1000.0;
> = 0;

 // Radial2 Pow
float R2pow
<
    string uiname="Radial2 Pow";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

// Pinwheel amount
float Pamn
<
    string uiname="Pinwheel";
    float uimin=0.0;
    float uimax=1000.0;
> = 0.0;

// Pinwheel Pow
float Ppow
<
    string uiname="Pinwheel Pow";
    float uimin=0.0;
    float uimax=1000.0;
> = 1.0;

// Multiplier
float Multiplier
<
    string uiname="Multiplier";
    float uimin=-1000.0;
    float uimax=1000.0;
> = 1.0;

// Phase
float Phase
<
    string uiname="Phase";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0;

// Linear Pow switch
bool LPe
<
    string uiname="Linear Pow enabled";
> = 0;

// Linear Pow switch
bool PPe
<
    string uiname="Pinwheel Pow enabled";
> = 0;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col = 0.0;
    float2 p1 = 0;
    float2 p2 = 0;
    float2 p3 = { 0.0,1.0 };
    float4 v = 0;
    float val,l,r,r2,p = 0.0;

    p2.x = In.TexCd.x;
    p2.y = In.TexCd.y;

    v.xy = p1;
    v.zw = p2;

    v -= 0.5;

    // linear
    l=p2.y;
    if (LPe) l = pow(l,Lpow);

    // radial
    r = distance(p1+0.5,p2);

    // Radial 2
    r2 = dot(v,v);

   // pinwheel
    p = 2.3873*atan2(0.5-p2.x,0.5-p2.y);  // 2.3873 ?
    if (PPe) p = pow(p,Ppow);
    
    //sum
    val = ((l*Lamn)+(pow(r,Rpow)*Ramn)+(pow(r2,R2pow)*R2amn)+(p*Pamn))*Multiplier;
    val = frac(val+Phase);

    if(val>0.5)
    {
        val -= 0.5;
        val *= -1.0;
        val += 0.5;
    }

    val *= 2;

    col = lerp(c1,c2,val);
    
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Gradients
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
